/*******************************************************************************************************
                            AMX JukeBox


  Author: KRoTaL
  Version: 0.1.1

  0.1    Release
  0.1.1  Added "stop wav" and "stop mp3" in the menu


  JukeBox Menu which supports wav and mp3 files.


  Command:

  amx_jukebox    -   displays jukebox menu
  say /jukebox   -   displays jukebox menu

  First page: choose wav or mp3
  Second Page: choose a sound
  Third Page: if wav, choose "spk" or "emit sound"
              if mp3, choose "mp3 play" or "mp3 loop"
  Fourth Page (only accessible to admins): choose a target (all players, team1, team2, or a specific player)


  Setup:

  Install the amx file.
  Enable VexdUM.

  Put the sound files in these folders:

  sound/jukebox/wav
  sound/jukebox/mp3


*******************************************************************************************************/

#include <amxmod>
#include <amxmisc>
#include <VexdUM>

#define ADMIN_ACCESS ADMIN_LEVEL_B

#define MAX_SOUNDS 50


new g_SoundFile[MAX_SOUNDS][2][64]
new g_SoundFileNum[2]
new g_wavDir[64]
new g_mp3Dir[64]
new g_wavPath[64]

new g_menuPosition[33]
new g_menuPlayers[33][32]
new g_menuPlayersNum[33]
new g_soundType[33]
new g_soundPlayType[33]
new g_soundNum[33]
new g_coloredMenus

public plugin_init()
{
  register_plugin("JukeBox", "0.1.1", "KRoTaL")
  register_concmd("amx_jukebox", "JukeBoxMenuCmd", 0, "- displays JukeBox menu")
  register_concmd("say /jukebox", "JukeBoxMenuCmd", 0, "- displays JukeBox menu")

  register_menucmd(register_menuid("JukeBox Wav/Mp3"), 1023, "actionWavMp3Menu")
  register_menucmd(register_menuid("JukeBox Sound List"), 1023, "actionSoundListMenu")
  register_menucmd(register_menuid("JukeBox Play Type"), 1023, "actionPlayTypeMenu")
  register_menucmd(register_menuid("JukeBox Targets"), 1023, "actionTargetsMenu")
  register_menucmd(register_menuid("JukeBox Players"), 1023, "actionPlayersMenu")
  g_coloredMenus = (is_running("cstrike") || is_running("czero") || is_running("dod"))
}

public plugin_precache()
{
  copy(g_wavDir, 63, "sound/jukebox/wav")
  copy(g_mp3Dir, 63, "sound/jukebox/mp3")
  g_SoundFileNum[0] = 0
  new len, pos = 0
  copy(g_wavPath, 63, g_wavDir)
  replace(g_wavPath, 63, "sound/", "")
  while(g_SoundFileNum[0] < MAX_SOUNDS && (pos = read_dir(g_wavDir, pos, g_SoundFile[g_SoundFileNum[0]][0], 63, len)))
  {
    format(g_SoundFile[g_SoundFileNum[0]][0], 63, "%s/%s", g_wavPath, g_SoundFile[g_SoundFileNum[0]][0])
    if(len < 4 || containi(g_SoundFile[g_SoundFileNum[0]][0], ".wav") == -1
      || containi(g_SoundFile[g_SoundFileNum[0]][0], ".ztmp") != -1) continue
    precache_sound(g_SoundFile[g_SoundFileNum[0]][0])
    g_SoundFileNum[0]++
  }
  g_SoundFileNum[1] = 0
  pos = 0
  while(g_SoundFileNum[1] < MAX_SOUNDS && (pos = read_dir(g_mp3Dir, pos, g_SoundFile[g_SoundFileNum[1]][1], 63, len)))
  {
    format(g_SoundFile[g_SoundFileNum[1]][1], 63, "%s/%s", g_mp3Dir, g_SoundFile[g_SoundFileNum[1]][1])
    if(len < 4 || containi(g_SoundFile[g_SoundFileNum[1]][1], ".mp3") == -1
      || containi(g_SoundFile[g_SoundFileNum[1]][1], ".ztmp") != -1) continue
    precache_generic(g_SoundFile[g_SoundFileNum[1]][1])
    g_SoundFileNum[1]++
  }
}

public JukeBoxMenuCmd(id)
{
  displayWavMp3Menu(id, g_menuPosition[id] = 0)
  return PLUGIN_HANDLED
}

public actionWavMp3Menu(id, key)
{
  switch(key)
  {
    case 0: {
      g_soundType[id] = 0
      displaySoundListMenu(id, g_menuPosition[id] = 0)
    }
    case 1: {
      g_soundType[id] = 1
      displaySoundListMenu(id, g_menuPosition[id] = 0)
    }
    case 2: {
      client_cmd(id, "stopsound")
    }
    case 3: {
      client_cmd(id, "mp3 stop")
    }
    default: return PLUGIN_HANDLED
  }

  return PLUGIN_HANDLED
}

displayWavMp3Menu(id,pos) {
  if(pos < 0)
    return

  if(!g_SoundFileNum[0] && !g_SoundFileNum[1])
  {
    client_print(id, print_chat, "[AMX JukeBox] No sounds in the JukeBox.")
    return
  }

  new menuBody[512]
  new b = 0

  new len = format(menuBody, 511, g_coloredMenus ?
    "\yJukeBox Wav/Mp3^n\w^n" :
    "JukeBox Wav/Mp3^n^n")

  new keys = (1<<9)

  if(g_SoundFileNum[0] > 0)
  {
    keys |= (1<<b)
    len += format(menuBody[len], 511-len, "%d. Wav^n", ++b)
  }
  else
  {
    if(g_coloredMenus)
      len += format(menuBody[len], 511-len, "\d%d. Wav^n\w", ++b)
    else
      len += format(menuBody[len], 511-len, "#%d. Wav^n", ++b)
  }

  if(g_SoundFileNum[1] > 0)
  {
    keys |= (1<<b)
    len += format(menuBody[len], 511-len, "%d. Mp3^n", ++b)
  }
  else
  {
    if(g_coloredMenus)
      len += format(menuBody[len], 511-len, "\d%d. Mp3^n\w", ++b)
    else
      len += format(menuBody[len], 511-len, "#%d. Mp3^n", ++b)
  }

  keys |= (1<<b)
  len += format(menuBody[len], 511-len, "%d. Stop Wav^n", ++b)

  keys |= (1<<b)
  len += format(menuBody[len], 511-len, "%d. Stop Mp3^n", ++b)

  format(menuBody[len], 511-len, "^n0. Exit")

  show_menu(id, keys, menuBody, -1, "JukeBox Wav/Mp3")
}

public actionTargetsMenu(id, key)
{
  new players[32], inum
  get_players(players, inum)
  switch(key)
  {
    case 0: {
      for(new i = 0; i < inum; i++)
      {
        playSound(id, players[i], g_soundNum[id])
      }
    }
    case 1: {
      for(new i = 0; i < inum; i++)
      {
        if(get_user_team(players[i]) == 1)
          playSound(id, players[i], g_soundNum[id])
      }
    }
    case 2: {
      for(new i = 0; i < inum; i++)
      {
        if(get_user_team(players[i]) == 2)
          playSound(id, players[i], g_soundNum[id])
      }
    }
    case 3: displayPlayersMenu(id, g_menuPosition[id] = 0)
    default: return PLUGIN_HANDLED
  }

  return PLUGIN_HANDLED
}

displayTargetsMenu(id,pos) {
  if(pos < 0)
    return

  new menuBody[512]
  new b = 0

  new len = format(menuBody, 511, g_coloredMenus ?
    "\yJukeBox Targets^n\w^n" :
    "JukeBox Targets^n^n")

  new keys = (1<<9)

  keys |= (1<<b)
  len += format(menuBody[len], 511-len, "%d. All players^n", ++b)

  keys |= (1<<b)
  len += format(menuBody[len], 511-len, "%d. Team 1^n", ++b)

  keys |= (1<<b)
  len += format(menuBody[len], 511-len, "%d. Team 2^n", ++b)

  keys |= (1<<b)
  len += format(menuBody[len], 511-len, "%d. Select Player^n", ++b)

  format(menuBody[len], 511-len, "^n0. Exit")

  show_menu(id, keys, menuBody, -1, "JukeBox Targets")
}

public actionPlayersMenu(id, key)
{
  switch(key)
  {
    case 8: displayPlayersMenu(id, ++g_menuPosition[id])
    case 9: displayPlayersMenu(id, --g_menuPosition[id])
    default: {
      playSound(id, g_menuPlayers[id][g_menuPosition[id] * 8 + key], g_soundNum[id])
    }
  }

  return PLUGIN_HANDLED
}

displayPlayersMenu(id,pos) {
  if(pos < 0)
    return

  get_players(g_menuPlayers[id], g_menuPlayersNum[id])

  new menuBody[512]
  new b = 0
  new i
  new name[32]
  new start = pos * 8

  if(start >= g_menuPlayersNum[id])
    start = pos = g_menuPosition[id] = 0

  new len = format(menuBody, 511, g_coloredMenus ?
    "\yJukeBox Players\R%d/%d^n\w^n" :
    "JukeBox Players %d/%d^n^n",
    pos+1, (g_menuPlayersNum[id] / 8 + ((g_menuPlayersNum[id] % 8) ? 1 : 0)))

  new end = start + 8
  new keys = (1<<9)

  if(end > g_menuPlayersNum[id])
    end = g_menuPlayersNum[id]

  for(new a = start; a < end; ++a) {
    i = g_menuPlayers[id][a]
    get_user_name(i, name, 31)
    if(get_user_flags(i) & ADMIN_IMMUNITY && i != id) {
      ++b
      if(g_coloredMenus)
        len += format(menuBody[len], 511-len, "\d%d. %s^n\w", b, name)
      else
        len += format(menuBody[len], 511-len, "#. %s^n", name)
    }
    else {
      keys |= (1<<b)
      len += format(menuBody[len], 511-len, "%d. %s^n", ++b, name)
    }
  }

  if(end != g_menuPlayersNum[id]) {
    format(menuBody[len], 511-len, "^n9. More...^n0. %s",
      pos ? "Back"
          : "Exit")
    keys |= (1<<8)
  }
  else
    format(menuBody[len], 511-len, "^n0. %s",
      pos ? "Back"
          : "Exit")

  show_menu(id, keys, menuBody, -1, "JukeBox Players")
}

public actionPlayTypeMenu(id, key)
{
  new players[32], inum
  get_players(players, inum)
  switch(key)
  {
    case 0: {
      g_soundPlayType[id] = 0
      if(get_user_flags(id) & ADMIN_ACCESS)
        displayTargetsMenu(id, g_menuPosition[id] = 0)
      else
        playSound(id, id, g_soundNum[id])
    }
    case 1: {
      g_soundPlayType[id] = 1
      if(get_user_flags(id) & ADMIN_ACCESS)
        displayTargetsMenu(id, g_menuPosition[id] = 0)
      else
        playSound(id, id, g_soundNum[id])
    }
    default: return PLUGIN_HANDLED
  }

  return PLUGIN_HANDLED
}

displayPlayTypeMenu(id,pos) {
  if(pos < 0)
    return

  new menuBody[512]
  new b = 0

  new len = format(menuBody, 511, g_coloredMenus ?
    "\yJukeBox Play Type^n\w^n" :
    "JukeBox Play Type ^n^n")

  new keys = (1<<9)

  keys |= (1<<b)
  if(g_soundType[id] == 0)
    len += format(menuBody[len], 511-len, "%d. Spk^n", ++b)
  else
    len += format(menuBody[len], 511-len, "%d. Mp3 Play^n", ++b)

  keys |= (1<<b)
  if(g_soundType[id] == 0)
    len += format(menuBody[len], 511-len, "%d. Emit Sound^n", ++b)
  else
    len += format(menuBody[len], 511-len, "%d. Mp3 Loop^n", ++b)

  format(menuBody[len], 511-len, "^n0. Exit")

  show_menu(id, keys, menuBody, -1, "JukeBox Play Type")
}

public actionSoundListMenu(id, key)
{
  switch(key)
  {
    case 8: displaySoundListMenu(id, ++g_menuPosition[id])
    case 9: displaySoundListMenu(id, --g_menuPosition[id])
    default:
    {
      g_soundNum[id] = g_menuPosition[id] * 8 + key
      displayPlayTypeMenu(id, g_menuPosition[id] = 0)
    }
  }

  return PLUGIN_HANDLED
}

displaySoundListMenu(id, pos)
{
  if (pos < 0)
    return

  new menuBody[512]
  new start = pos * 8
  new b = 0

  if (start >= g_SoundFileNum[g_soundType[id]])
    start = pos = g_menuPosition[id] = 0

  new len = format(menuBody,511, g_coloredMenus ?
    "\yJukeBox Sound List\R%d/%d^n\w^n" :
    "JukeBox Sound List %d/%d^n^n",
    pos+1,(g_SoundFileNum[g_soundType[id]] / 8 + ((g_SoundFileNum[g_soundType[id]] % 8) ? 1 : 0 )) )

  new end = start + 8
  new keys = (1<<9)

  if (end > g_SoundFileNum[g_soundType[id]])
    end = g_SoundFileNum[g_soundType[id]]

  for(new a = start ; a < end ; ++a)
  {
    keys |= (1<<b)
    new sound[64]
    copy(sound, 63, g_SoundFile[a][g_soundType[id]])
    replace(sound, 63, g_wavPath, "")
    replace(sound, 63, ".wav", "")
    replace(sound, 63, g_mp3Dir, "")
    replace(sound, 63, ".mp3", "")
    if(sound[0] == '/') replace(sound, 63, "/", "")
    len += format(menuBody[len],511-len,"%d. %s^n", ++b, sound)
  }

  if (end != g_SoundFileNum[g_soundType[id]])
  {
    format(menuBody[len],511-len,"^n9. More...^n0. %s", pos ? "Back" : "Exit", id)
    keys |= (1<<8)
  }
  else
  {
    format(menuBody[len],511-len,"^n0. %s", pos ? "Back" : "Exit", id)
  }

  show_menu(id, keys, menuBody, -1, "JukeBox Sound List")

  return
}

playSound(id, target, sndNum)
{
  if(g_soundType[id] == 0)
  {
    if(g_soundPlayType[id] == 0)
    {
      client_cmd(target, "spk %s", g_SoundFile[sndNum][g_soundType[id]])
    }
    else if(g_soundPlayType[id] == 1)
    {
      emit_sound(target, CHAN_BODY, g_SoundFile[sndNum][g_soundType[id]], 1.0, ATTN_NORM, 0, PITCH_NORM)
    }
  }
  else if(g_soundType[id] == 1)
  {
    if(g_soundPlayType[id] == 0)
    {
      client_cmd(target, "mp3 play ^"%s^"", g_SoundFile[sndNum][g_soundType[id]])
    }
    else if(g_soundPlayType[id] == 1)
    {
      client_cmd(target, "mp3 loop ^"%s^"", g_SoundFile[sndNum][g_soundType[id]])
    }
  }
}
